@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/clients/tools/ldappasswd.c,v 1.32.4.19 2001/03/15 03:11:07 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/clients/tools/ldappasswd.c,v 1.32.4.20 2001/08/28 17:14:48 kurt Exp $ */
 /*
  * Copyright 1998-2000 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -596,7 +596,7 @@ main( int argc, char *argv[] )
 		return EXIT_FAILURE;
 	}
 
-	if ( use_tls && ldap_start_tls_s( ld, NULL, NULL ) != LDAP_SUCCESS ) {
+	if ( use_tls && ( ldap_start_tls_s( ld, NULL, NULL ) != LDAP_SUCCESS )) {
 		ldap_perror( ld, "ldap_start_tls" );
 		if ( use_tls > 1 ) {
 			return( EXIT_FAILURE );
