@@ -1,5 +1,5 @@
 /* io.c - ber general i/o routines */
-/* $OpenLDAP: pkg/ldap/libraries/liblber/io.c,v 1.38.4.5 2000/10/12 19:52:40 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/liblber/io.c,v 1.38.4.7 2001/05/10 04:43:46 kurt Exp $ */
 /*
  * Copyright 1998-2000 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -77,7 +77,7 @@ ber_read(
 
 	assert( BER_VALID( ber ) );
 
-	nleft = ber->ber_end - ber->ber_ptr;
+	nleft = ber_pvt_ber_remaining( ber );
 	actuallen = nleft < len ? nleft : len;
 
 	AC_MEMCPY( buf, ber->ber_ptr, actuallen );
@@ -133,13 +133,14 @@ ber_realloc( BerElement *ber, ber_len_t len )
 
 	assert( BER_VALID( ber ) );
 
-	have = (ber->ber_end - ber->ber_buf) / EXBUFSIZ;
+	total = ber_pvt_ber_total( ber );
+	have = total / EXBUFSIZ;
 	need = (len < EXBUFSIZ ? 1 : (len + (EXBUFSIZ - 1)) / EXBUFSIZ);
 	total = have * EXBUFSIZ + need * EXBUFSIZ;
 
 	oldbuf = ber->ber_buf;
 
-	ber->ber_buf = (char *) LBER_REALLOC( ber->ber_buf, total );
+	ber->ber_buf = (char *) LBER_REALLOC( oldbuf, total );
 	
 	if ( ber->ber_buf == NULL ) {
 		ber->ber_buf = oldbuf;
@@ -251,8 +252,9 @@ ber_alloc_t( int options )
 
 	ber = (BerElement *) LBER_CALLOC( 1, sizeof(BerElement) );
 
-	if ( ber == NULL )
-		return( NULL );
+	if ( ber == NULL ) {
+		return NULL;
+	}
 
 	ber->ber_valid = LBER_VALID_BERELEMENT;
 	ber->ber_tag = LBER_DEFAULT;
@@ -260,19 +262,19 @@ ber_alloc_t( int options )
 	ber->ber_debug = ber_int_debug;
 
 	assert( BER_VALID( ber ) );
-	return( ber );
+	return ber;
 }
 
 BerElement *
 ber_alloc( void )	/* deprecated */
 {
-	return( ber_alloc_t( 0 ) );
+	return ber_alloc_t( 0 );
 }
 
 BerElement *
 der_alloc( void )	/* deprecated */
 {
-	return( ber_alloc_t( LBER_USE_DER ) );
+	return ber_alloc_t( LBER_USE_DER );
 }
 
 BerElement *
@@ -284,7 +286,7 @@ ber_dup( BerElement *ber )
 	assert( BER_VALID( ber ) );
 
 	if ( (new = ber_alloc_t( ber->ber_options )) == NULL ) {
-		return( NULL );
+		return NULL;
 	}
 
 	*new = *ber;
@@ -367,7 +369,7 @@ int ber_flatten(
 		return -1;
 	}
 
-	bv = LBER_MALLOC( sizeof(struct berval));
+	bv = LBER_MALLOC( sizeof(struct berval) );
 	if ( bv == NULL ) {
 		return -1;
 	}
@@ -379,7 +381,7 @@ int ber_flatten(
 
 	} else {
 		/* copy the berval */
-		ber_len_t len = ber->ber_ptr - ber->ber_buf;
+		ber_len_t len = ber_pvt_ber_write( ber );
 
 		bv->bv_val = (char *) LBER_MALLOC( len + 1 );
 		if ( bv->bv_val == NULL ) {
@@ -405,6 +407,7 @@ ber_reset( BerElement *ber, int was_writing )
 	if ( was_writing ) {
 		ber->ber_end = ber->ber_ptr;
 		ber->ber_ptr = ber->ber_buf;
+
 	} else {
 		ber->ber_ptr = ber->ber_end;
 	}
@@ -463,8 +466,10 @@ ber_get_next(
 		if (ber->ber_rwptr == (char *) &ber->ber_tag) {
 			if (ber_int_sb_read( sb, ber->ber_rwptr, 1)<=0)
 				return LBER_DEFAULT;
+
 			if ((ber->ber_rwptr[0] & LBER_BIG_TAG_MASK)
-				!= LBER_BIG_TAG_MASK) {
+				!= LBER_BIG_TAG_MASK)
+			{
 				ber->ber_tag = ber->ber_rwptr[0];
 				ber->ber_rwptr = (char *) &ber->ber_usertag;
 				goto get_lenbyte;
@@ -492,17 +497,21 @@ ber_get_next(
 get_lenbyte:
 	if (ber->ber_rwptr==(char *) &ber->ber_usertag) {
 		unsigned char c;
-		if (ber_int_sb_read( sb, (char *) &c, 1)<=0)
+		if (ber_int_sb_read( sb, (char *) &c, 1)<=0) {
 			return LBER_DEFAULT;
+		}
+
 		if (c & 0x80U) {
 			int len = c & 0x7fU;
 			if ( (len==0) || ( len>sizeof( ber->ber_len ) ) ) {
 				errno = ERANGE;
 				return LBER_DEFAULT;
 			}
+
 			ber->ber_rwptr = (char *) &ber->ber_len +
 				sizeof(ber->ber_len) - len;
 			ber->ber_len = 0;
+
 		} else {
 			ber->ber_len = c;
 			goto fill_buffer;
@@ -517,6 +526,7 @@ get_lenbyte:
 		to_go = (char *) &ber->ber_len + sizeof( ber->ber_len ) -
 			ber->ber_rwptr;
 		assert( to_go > 0 );
+
 		res = BerRead( sb, netlen, to_go );
 		if (res <= 0) {
 			return LBER_DEFAULT;
@@ -524,15 +534,27 @@ get_lenbyte:
 		ber->ber_rwptr += res;
 
 		/* convert length. */
-		ber->ber_len = 0;
 		for( to_go = 0; to_go < res ; to_go++ ) {
 			ber->ber_len <<= 8;
 			ber->ber_len |= netlen[to_go];
 		}
+
+		if (PTR_IN_VAR(ber->ber_rwptr, ber->ber_len)) {
+			return LBER_DEFAULT;
+		}
 	}
 
 fill_buffer:	
 	/* now fill the buffer. */
+
+	/* make sure length is reasonable */
+	if ( ber->ber_len == 0 ||
+		( sb->sb_max_incoming && ber->ber_len > sb->sb_max_incoming ))
+	{
+		errno = ERANGE;
+		return LBER_DEFAULT;
+	}
+
 	if (ber->ber_buf==NULL) {
 		ber->ber_buf = (char *) LBER_MALLOC( ber->ber_len );
 		if (ber->ber_buf==NULL) {
@@ -574,6 +596,7 @@ fill_buffer:
 		}
 		return (ber->ber_tag);
 	}
+
 	assert( 0 ); /* ber structure is messed up ?*/
 	return LBER_DEFAULT;
 }
