@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/init.c,v 1.33.2.8 2001/07/21 19:01:39 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/init.c,v 1.33.2.9 2001/09/07 21:17:03 kurt Exp $ */
 /*
  * Copyright 1998-2000 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -221,7 +221,7 @@ static void openldap_ldap_init_w_conf(
 				break;
 			case ATTR_TLS:
 #ifdef HAVE_TLS
-			   	ldap_int_tls_config( gopts, attrs[i].offset, opt );
+			   	ldap_int_tls_config( NULL, attrs[i].offset, opt );
 #endif
 				break;
 			}
@@ -357,7 +357,7 @@ static void openldap_ldap_init_w_env(
 		   	break;
 		case ATTR_TLS:
 #ifdef HAVE_TLS
-		   	ldap_int_tls_config( gopts, attrs[i].offset, value );
+		   	ldap_int_tls_config( NULL, attrs[i].offset, value );
 #endif			 	
 		   	break;
 		}
