@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/test.c,v 1.20.6.4 2000/08/18 02:32:54 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/test.c,v 1.20.6.5 2001/07/21 19:01:40 kurt Exp $ */
 /*
  * Copyright 1998-2000 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -599,37 +599,6 @@ main( int argc, char **argv )
 			timeout.tv_sec = atoi( line );
 			break;
 
-		case 'U':	/* set ufn search prefix */
-			getline( line, sizeof(line), stdin, "ufn prefix? " );
-			ldap_ufn_setprefix( ld, line );
-			break;
-
-		case 'u':	/* user friendly search w/optional timeout */
-			getline( dn, sizeof(dn), stdin, "ufn? " );
-			strcat( dn, dnsuffix );
-			types = get_list( "attrs to return? " );
-			getline( line, sizeof(line), stdin,
-			    "attrsonly (0=attrs&values, 1=attrs only)? " );
-			attrsonly = atoi( line );
-
-			if ( command2 == 't' ) {
-				id = ldap_ufn_search_c( ld, dn, types,
-				    attrsonly, &res, ldap_ufn_timeout,
-				    &timeout );
-			} else {
-				id = ldap_ufn_search_s( ld, dn, types,
-				    attrsonly, &res );
-			}
-			if ( res == NULL )
-				ldap_perror( ld, "ldap_ufn_search" );
-			else {
-				printf( "\nresult: err %d\n", id );
-				handle_result( ld, res );
-				res = NULL;
-			}
-			free_list( types );
-			break;
-
 		case 'l':	/* URL search */
 			getline( line, sizeof(line), stdin,
 			    "attrsonly (0=attrs&values, 1=attrs only)? " );
